@@ -1,28 +1,35 @@
 /* nettle-benchmark.c
- *
- * Tries the performance of the various algorithms.
- *
- */
- 
-/* nettle, low-level cryptographics library
- *
- * Copyright (C) 2001, 2010 Niels Mller
- *  
- * The nettle library is free software; you can redistribute it and/or modify
- * it under the terms of the GNU Lesser General Public License as published by
- * the Free Software Foundation; either version 2.1 of the License, or (at your
- * option) any later version.
- * 
- * The nettle library is distributed in the hope that it will be useful, but
- * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
- * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public
- * License for more details.
- * 
- * You should have received a copy of the GNU Lesser General Public License
- * along with the nettle library; see the file COPYING.LIB.  If not, write to
- * the Free Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
- * MA 02111-1307, USA.
- */
+   
+   Tests the performance of the various algorithms.
+
+   Copyright (C) 2001, 2010, 2014 Niels Möller
+
+   This file is part of GNU Nettle.
+
+   GNU Nettle is free software: you can redistribute it and/or
+   modify it under the terms of either:
+
+     * the GNU Lesser General Public License as published by the Free
+       Software Foundation; either version 3 of the License, or (at your
+       option) any later version.
+
+   or
+
+     * the GNU General Public License as published by the Free
+       Software Foundation; either version 2 of the License, or (at your
+       option) any later version.
+
+   or both in parallel, as here.
+
+   GNU Nettle is distributed in the hope that it will be useful,
+   but WITHOUT ANY WARRANTY; without even the implied warranty of
+   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+   General Public License for more details.
+
+   You should have received copies of the GNU General Public License and
+   the GNU Lesser General Public License along with this program.  If
+   not, see http://www.gnu.org/licenses/.
+*/
 
 #if HAVE_CONFIG_H
 # include "config.h"
@@ -38,6 +45,8 @@
 
 #include <time.h>
 
+#include "timing.h"
+
 #include "aes.h"
 #include "arcfour.h"
 #include "blowfish.h"
@@ -45,11 +54,17 @@
 #include "cbc.h"
 #include "ctr.h"
 #include "des.h"
+#include "eax.h"
 #include "gcm.h"
 #include "memxor.h"
+#include "salsa20.h"
 #include "serpent.h"
-#include "sha.h"
+#include "sha1.h"
+#include "sha2.h"
+#include "sha3.h"
 #include "twofish.h"
+#include "umac.h"
+#include "poly1305.h"
 
 #include "nettle-meta.h"
 #include "nettle-internal.h"
@@ -96,7 +111,7 @@ static double frequency = 0.0;
 #define BENCH_ITERATIONS 10
 #endif
 
-static void
+static void NORETURN PRINTF_STYLE(1,2)
 die(const char *format, ...)
 {
   va_list args;
@@ -109,58 +124,6 @@ die(const char *format, ...)
 
 static double overhead = 0.0; 
 
-#if HAVE_CLOCK_GETTIME && defined CLOCK_PROCESS_CPUTIME_ID
-#define TRY_CLOCK_GETTIME 1
-struct timespec cgt_start;
-
-static int
-cgt_works_p(void)
-{
-  struct timespec now;
-  return clock_gettime(CLOCK_PROCESS_CPUTIME_ID, &now) == 0;
-}
-
-static void
-cgt_time_start(void)
-{
-  if (clock_gettime(CLOCK_PROCESS_CPUTIME_ID, &cgt_start) < 0)
-    die("clock_gettime failed: %s\n", strerror(errno));
-}
-
-static double
-cgt_time_end(void)
-{
-    struct timespec end;
-    if (clock_gettime(CLOCK_PROCESS_CPUTIME_ID, &end) < 0)
-      die("clock_gettime failed: %s\n", strerror(errno));
-
-    return end.tv_sec - cgt_start.tv_sec
-      + 1e-9 * (end.tv_nsec - cgt_start.tv_nsec);
-}
-
-static void (*time_start)(void);
-static double (*time_end)(void);
-
-#else /* !HAVE_CLOCK_GETTIME */
-#define TRY_CLOCK_GETTIME 0
-#define time_start clock_time_start
-#define time_end clock_time_end
-#endif /* !HAVE_CLOCK_GETTIME */
-
-static clock_t clock_start;
-
-static void
-clock_time_start(void)
-{
-  clock_start = clock();
-}
-
-static double
-clock_time_end(void)
-{
-  return (double) (clock() - (clock_start)) / CLOCKS_PER_SEC;
-}
-
 /* Returns second per function call */
 static double
 time_function(void (*f)(void *arg), void *arg)
@@ -230,7 +193,7 @@ bench_hash(void *arg)
 struct bench_cipher_info
 {
   void *ctx;
-  nettle_crypt_func *crypt;
+  nettle_cipher_func *crypt;
   uint8_t *data;
 };
 
@@ -244,7 +207,7 @@ bench_cipher(void *arg)
 struct bench_cbc_info
 {
   void *ctx;
-  nettle_crypt_func *crypt;
+  nettle_cipher_func *crypt;
  
   uint8_t *data;
   
@@ -279,6 +242,28 @@ bench_ctr(void *arg)
 	    BENCH_BLOCK, info->data, info->data);
 }
 
+struct bench_aead_info
+{
+  void *ctx;
+  nettle_crypt_func *crypt;
+  nettle_hash_update_func *update;
+  uint8_t *data;
+};
+
+static void
+bench_aead_crypt(void *arg)
+{
+  const struct bench_aead_info *info = arg;
+  info->crypt (info->ctx, BENCH_BLOCK, info->data, info->data);
+}
+
+static void
+bench_aead_update(void *arg)
+{
+  const struct bench_aead_info *info = arg;
+  info->update (info->ctx, BENCH_BLOCK, info->data);
+}
+
 /* Set data[i] = floor(sqrt(i)) */
 static void
 init_data(uint8_t *data)
@@ -301,6 +286,15 @@ init_key(unsigned length,
     key[i] = i;
 }
 
+static void
+init_nonce(unsigned length,
+	   uint8_t *nonce)
+{
+  unsigned i;
+  for (i = 0; i<length; i++)
+    nonce[i] = 3*i;
+}
+
 static void
 header(void)
 {
@@ -400,37 +394,82 @@ time_hash(const struct nettle_hash *hash)
 }
 
 static void
-time_gcm(void)
+time_umac(void)
 {
   static uint8_t data[BENCH_BLOCK];
-  struct bench_hash_info hinfo;
-  struct bench_cipher_info cinfo;
-  struct gcm_aes_ctx ctx;
-
+  struct bench_hash_info info;
+  struct umac32_ctx ctx32;
+  struct umac64_ctx ctx64;
+  struct umac96_ctx ctx96;
+  struct umac128_ctx ctx128;
+  
   uint8_t key[16];
-  uint8_t iv[GCM_IV_SIZE];
 
-  gcm_aes_set_key(&ctx, sizeof(key), key);
-  gcm_aes_set_iv(&ctx, sizeof(iv), iv);
+  umac32_set_key (&ctx32, key);
+  info.ctx = &ctx32;
+  info.update = (nettle_hash_update_func *) umac32_update;
+  info.data = data;
 
-  hinfo.ctx = &ctx;
-  hinfo.update = (nettle_hash_update_func *) gcm_aes_update;
-  hinfo.data = data;
-  
-  display("gcm-aes", "update", GCM_BLOCK_SIZE,
-	  time_function(bench_hash, &hinfo));
-  
-  cinfo.ctx = &ctx;
-  cinfo.crypt = (nettle_crypt_func *) gcm_aes_encrypt;
-  cinfo.data = data;
+  display("umac32", "update", UMAC_BLOCK_SIZE,
+	  time_function(bench_hash, &info));
 
-  display("gcm-aes", "encrypt", GCM_BLOCK_SIZE,
-	  time_function(bench_cipher, &cinfo));
+  umac64_set_key (&ctx64, key);
+  info.ctx = &ctx64;
+  info.update = (nettle_hash_update_func *) umac64_update;
+  info.data = data;
 
-  cinfo.crypt = (nettle_crypt_func *) gcm_aes_decrypt;
+  display("umac64", "update", UMAC_BLOCK_SIZE,
+	  time_function(bench_hash, &info));
 
-  display("gcm-aes", "decrypt", GCM_BLOCK_SIZE,
-	  time_function(bench_cipher, &cinfo));
+  umac96_set_key (&ctx96, key);
+  info.ctx = &ctx96;
+  info.update = (nettle_hash_update_func *) umac96_update;
+  info.data = data;
+
+  display("umac96", "update", UMAC_BLOCK_SIZE,
+	  time_function(bench_hash, &info));
+
+  umac128_set_key (&ctx128, key);
+  info.ctx = &ctx128;
+  info.update = (nettle_hash_update_func *) umac128_update;
+  info.data = data;
+
+  display("umac128", "update", UMAC_BLOCK_SIZE,
+	  time_function(bench_hash, &info));
+}
+
+static void
+time_poly1305_aes(void)
+{
+  static uint8_t data[BENCH_BLOCK];
+  struct bench_hash_info info;
+  struct poly1305_aes_ctx ctx;
+  uint8_t key[32];
+
+  poly1305_aes_set_key (&ctx, key);
+  info.ctx = &ctx;
+  info.update = (nettle_hash_update_func *) poly1305_aes_update;
+  info.data = data;
+
+  display("poly1305-aes", "update", 1024,
+	  time_function(bench_hash, &info));
+}
+
+static int
+prefix_p(const char *prefix, const char *s)
+{
+  size_t i;
+  for (i = 0; prefix[i]; i++)
+    if (prefix[i] != s[i])
+      return 0;
+  return 1;
+}
+
+static int
+block_cipher_p(const struct nettle_cipher *cipher)
+{
+  /* Don't use nettle cbc and ctr for openssl ciphers. */
+  return cipher->block_size > 0 && !prefix_p("openssl", cipher->name);
 }
 
 static void
@@ -453,7 +492,7 @@ time_cipher(const struct nettle_cipher *cipher)
     info.data = data;
     
     init_key(cipher->key_size, key);
-    cipher->set_encrypt_key(ctx, cipher->key_size, key);
+    cipher->set_encrypt_key(ctx, key);
 
     display(cipher->name, "ECB encrypt", cipher->block_size,
 	    time_function(bench_cipher, &info));
@@ -466,14 +505,13 @@ time_cipher(const struct nettle_cipher *cipher)
     info.data = data;
     
     init_key(cipher->key_size, key);
-    cipher->set_decrypt_key(ctx, cipher->key_size, key);
+    cipher->set_decrypt_key(ctx, key);
 
     display(cipher->name, "ECB decrypt", cipher->block_size,
 	    time_function(bench_cipher, &info));
   }
 
-  /* Don't use nettle cbc to benchmark openssl ciphers */
-  if (cipher->block_size && cipher->name[0] != 'o')
+  if (block_cipher_p(cipher))
     {
       uint8_t *iv = xalloc(cipher->block_size);
       
@@ -486,9 +524,9 @@ time_cipher(const struct nettle_cipher *cipher)
 	info.block_size = cipher->block_size;
 	info.iv = iv;
     
-        memset(iv, 0, sizeof(iv));
+        memset(iv, 0, sizeof(cipher->block_size));
     
-        cipher->set_encrypt_key(ctx, cipher->key_size, key);
+        cipher->set_encrypt_key(ctx, key);
 
 	display(cipher->name, "CBC encrypt", cipher->block_size,
 		time_function(bench_cbc_encrypt, &info));
@@ -502,9 +540,9 @@ time_cipher(const struct nettle_cipher *cipher)
 	info.block_size = cipher->block_size;
 	info.iv = iv;
     
-        memset(iv, 0, sizeof(iv));
+        memset(iv, 0, sizeof(cipher->block_size));
 
-        cipher->set_decrypt_key(ctx, cipher->key_size, key);
+        cipher->set_decrypt_key(ctx, key);
 
 	display(cipher->name, "CBC decrypt", cipher->block_size,
 		time_function(bench_cbc_decrypt, &info));
@@ -519,9 +557,9 @@ time_cipher(const struct nettle_cipher *cipher)
 	info.block_size = cipher->block_size;
 	info.iv = iv;
     
-        memset(iv, 0, sizeof(iv));
+        memset(iv, 0, sizeof(cipher->block_size));
     
-        cipher->set_encrypt_key(ctx, cipher->key_size, key);
+        cipher->set_encrypt_key(ctx, key);
 
 	display(cipher->name, "CTR", cipher->block_size,
 		time_function(bench_ctr, &info));	
@@ -533,6 +571,73 @@ time_cipher(const struct nettle_cipher *cipher)
   free(key);
 }
 
+static void
+time_aead(const struct nettle_aead *aead)
+{
+  void *ctx = xalloc(aead->context_size);
+  uint8_t *key = xalloc(aead->key_size);
+  uint8_t *nonce = xalloc(aead->nonce_size);
+  static uint8_t data[BENCH_BLOCK];
+
+  printf("\n");
+  
+  init_data(data);
+  if (aead->set_nonce)
+    init_nonce (aead->nonce_size, nonce);
+
+  {
+    /* Decent initializers are a GNU extension, so don't use it here. */
+    struct bench_aead_info info;
+    info.ctx = ctx;
+    info.crypt = aead->encrypt;
+    info.data = data;
+    
+    init_key(aead->key_size, key);
+    aead->set_encrypt_key(ctx, key);
+    if (aead->set_nonce)
+      aead->set_nonce (ctx, nonce);
+
+    display(aead->name, "encrypt", aead->block_size,
+	    time_function(bench_aead_crypt, &info));
+  }
+  
+  {
+    struct bench_aead_info info;
+    info.ctx = ctx;
+    info.crypt = aead->decrypt;
+    info.data = data;
+    
+    init_key(aead->key_size, key);
+    aead->set_decrypt_key(ctx, key);
+    if (aead->set_nonce)
+      aead->set_nonce (ctx, nonce);
+
+    display(aead->name, "decrypt", aead->block_size,
+	    time_function(bench_aead_crypt, &info));
+  }
+
+  if (aead->update)
+    {
+      struct bench_aead_info info;
+      info.ctx = ctx;
+      info.update = aead->update;
+      info.data = data;
+
+      aead->set_encrypt_key(ctx, key);
+
+      if (aead->set_nonce)
+	aead->set_nonce (ctx, nonce);
+    
+      display(aead->name, "update", aead->block_size,
+	      time_function(bench_aead_update, &info));
+    }
+  free(ctx);
+  free(key);
+  free(nonce);
+}
+
+/* Try to get accurate cycle times for assembler functions. */
+#if WITH_CYCLE_COUNTER
 static int
 compare_double(const void *ap, const void *bp)
 {
@@ -546,41 +651,65 @@ compare_double(const void *ap, const void *bp)
     return 0;
 }
 
-/* Try to get accurate cycle times for assembler functions. */
+#define TIME_CYCLES(t, code) do {				\
+  double tc_count[5];						\
+  uint32_t tc_start_lo, tc_start_hi, tc_end_lo, tc_end_hi;	\
+  unsigned tc_i, tc_j;						\
+  for (tc_j = 0; tc_j < 5; tc_j++)				\
+    {								\
+      tc_i = 0;							\
+      GET_CYCLE_COUNTER(tc_start_hi, tc_start_lo);		\
+      for (; tc_i < BENCH_ITERATIONS; tc_i++)			\
+	{ code; }						\
+								\
+      GET_CYCLE_COUNTER(tc_end_hi, tc_end_lo);			\
+								\
+      tc_end_hi -= (tc_start_hi + (tc_start_lo > tc_end_lo));	\
+      tc_end_lo -= tc_start_lo;					\
+								\
+      tc_count[tc_j] = ldexp(tc_end_hi, 32) + tc_end_lo;	\
+    }								\
+  qsort(tc_count, 5, sizeof(double), compare_double);		\
+  (t) = tc_count[2] / BENCH_ITERATIONS;				\
+} while (0)
+
 static void
 bench_sha1_compress(void)
 {
-#if WITH_CYCLE_COUNTER
   uint32_t state[_SHA1_DIGEST_LENGTH];
-  uint8_t data[BENCH_ITERATIONS * SHA1_DATA_SIZE];
-  uint32_t start_lo, start_hi, end_lo, end_hi;
+  uint8_t data[SHA1_BLOCK_SIZE];
+  double t;
 
-  double count[5];
-  
-  uint8_t *p;
-  unsigned i, j;
+  TIME_CYCLES (t, _nettle_sha1_compress(state, data));
 
-  for (j = 0; j < 5; j++)
-    {
-      i = 0;
-      p = data;
-      GET_CYCLE_COUNTER(start_hi, start_lo);
-      for (; i < BENCH_ITERATIONS; i++, p += SHA1_DATA_SIZE)
-	_nettle_sha1_compress(state, p);
-
-      GET_CYCLE_COUNTER(end_hi, end_lo);
-
-      end_hi -= (start_hi + (start_lo > end_lo));
-      end_lo -= start_lo;
-
-      count[j] = ldexp(end_hi, 32) + end_lo;
-    }
-
-  qsort(count, 5, sizeof(double), compare_double);
-  printf("sha1_compress: %.2f cycles\n\n", count[2] / BENCH_ITERATIONS);  
-#endif
+  printf("sha1_compress: %.2f cycles\n", t);  
 }
 
+static void
+bench_salsa20_core(void)
+{
+  uint32_t state[_SALSA20_INPUT_LENGTH];
+  double t;
+
+  TIME_CYCLES (t, _nettle_salsa20_core(state, state, 20));
+  printf("salsa20_core: %.2f cycles\n", t);  
+}
+
+static void
+bench_sha3_permute(void)
+{
+  struct sha3_state state;
+  double t;
+
+  TIME_CYCLES (t, sha3_permute (&state));
+  printf("sha3_permute: %.2f cycles (%.2f / round)\n", t, t / 24.0);
+}
+#else
+#define bench_sha1_compress()
+#define bench_salsa20_core()
+#define bench_sha3_permute()
+#endif
+
 #if WITH_OPENSSL
 # define OPENSSL(x) x,
 #else
@@ -601,7 +730,10 @@ main(int argc, char **argv)
       &nettle_sha1, OPENSSL(&nettle_openssl_sha1)
       &nettle_sha224, &nettle_sha256,
       &nettle_sha384, &nettle_sha512,
-      &nettle_ripemd160,
+      &nettle_sha512_224, &nettle_sha512_256,
+      &nettle_sha3_224, &nettle_sha3_256,
+      &nettle_sha3_384, &nettle_sha3_512,
+      &nettle_ripemd160, &nettle_gosthash94,
       NULL
     };
 
@@ -611,7 +743,6 @@ main(int argc, char **argv)
       OPENSSL(&nettle_openssl_aes128)
       OPENSSL(&nettle_openssl_aes192)
       OPENSSL(&nettle_openssl_aes256)
-      &nettle_arcfour128, OPENSSL(&nettle_openssl_arcfour128)
       &nettle_blowfish128, OPENSSL(&nettle_openssl_blowfish128)
       &nettle_camellia128, &nettle_camellia192, &nettle_camellia256,
       &nettle_cast128, OPENSSL(&nettle_openssl_cast128)
@@ -622,6 +753,22 @@ main(int argc, char **argv)
       NULL
     };
 
+  const struct nettle_aead *aeads[] =
+    {
+      /* Stream ciphers */
+      &nettle_arcfour128, OPENSSL(&nettle_openssl_arcfour128)
+      &nettle_salsa20, &nettle_salsa20r12, &nettle_chacha,
+      /* Proper AEAD algorithme. */
+      &nettle_gcm_aes128,
+      &nettle_gcm_aes192,
+      &nettle_gcm_aes256,
+      &nettle_gcm_camellia128,
+      &nettle_gcm_camellia256,
+      &nettle_eax_aes128,
+      &nettle_chacha_poly1305,
+      NULL
+    };
+
   enum { OPT_HELP = 300 };
   static const struct option options[] =
     {
@@ -652,22 +799,11 @@ main(int argc, char **argv)
 
   alg = argv[optind];
 
-  /* Choose timing function */
-#if TRY_CLOCK_GETTIME
-  if (cgt_works_p())
-    {
-      time_start = cgt_time_start;
-      time_end = cgt_time_end;
-    }
-  else
-    {
-      fprintf(stderr, "clock_gettime not working, falling back to clock\n");
-      time_start = clock_time_start;
-      time_end = clock_time_end;
-    }
-#endif
+  time_init();
   bench_sha1_compress();
-
+  bench_salsa20_core();
+  bench_sha3_permute();
+  printf("\n");
   time_overhead();
 
   header();
@@ -682,15 +818,19 @@ main(int argc, char **argv)
     if (!alg || strstr(hashes[i]->name, alg))
       time_hash(hashes[i]);
 
+  if (!alg || strstr ("umac", alg))
+    time_umac();
+
+  if (!alg || strstr ("poly1305-aes", alg))
+    time_poly1305_aes();
+
   for (i = 0; ciphers[i]; i++)
     if (!alg || strstr(ciphers[i]->name, alg))
       time_cipher(ciphers[i]);
 
-  if (!alg || strstr ("gcm", alg))
-    {
-      printf("\n");
-      time_gcm();
-    }
+  for (i = 0; aeads[i]; i++)
+    if (!alg || strstr(aeads[i]->name, alg))
+      time_aead(aeads[i]);
 
   return 0;
 }
