@@ -18,12 +18,16 @@
 
 #if WITH_HOGWEED
 # include "rsa.h"
-# include "dsa.h"
+# include "dsa-compat.h"
 # include "ecc-curve.h"
 # include "ecc.h"
 # include "ecc-internal.h"
 # include "ecdsa.h"
 # include "gmp-glue.h"
+
+/* Undo dsa-compat name mangling */
+#undef dsa_generate_keypair
+#define dsa_generate_keypair nettle_dsa_generate_keypair
 #endif
 
 #include "nettle-meta.h"
@@ -43,18 +47,18 @@ xalloc(size_t size);
 
 struct tstring {
   struct tstring *next;
-  unsigned length;
+  size_t length;
   uint8_t data[1];
 };
 
 struct tstring *
-tstring_alloc (unsigned length);
+tstring_alloc (size_t length);
 
 void
 tstring_clear(void);
 
 struct tstring *
-tstring_data(unsigned length, const char *data);
+tstring_data(size_t length, const char *data);
 
 struct tstring *
 tstring_hex(const char *hex);
@@ -65,7 +69,7 @@ tstring_print_hex(const struct tstring *s);
 /* Decodes a NUL-terminated hex string. */
 
 void
-print_hex(unsigned length, const uint8_t *data);
+print_hex(size_t length, const uint8_t *data);
 
 /* The main program */
 void
@@ -131,11 +135,12 @@ test_cipher_stream(const struct nettle_cipher *cipher,
 
 void
 test_aead(const struct nettle_aead *aead,
+	  nettle_hash_update_func *set_nonce,
 	  const struct tstring *key,
 	  const struct tstring *authtext,
 	  const struct tstring *cleartext,
 	  const struct tstring *ciphertext,
-	  const struct tstring *iv,
+	  const struct tstring *nonce,
 	  const struct tstring *digest);
 
 void
@@ -145,13 +150,13 @@ test_hash(const struct nettle_hash *hash,
 
 void
 test_hash_large(const struct nettle_hash *hash,
-		unsigned count, unsigned length,
+		size_t count, size_t length,
 		uint8_t c,
 		const struct tstring *digest);
 
 void
 test_armor(const struct nettle_armor *armor,
-           unsigned data_length,
+           size_t data_length,
            const uint8_t *data,
            const uint8_t *ascii);
 
@@ -197,9 +202,25 @@ test_dsa256(const struct dsa_public_key *pub,
 	    const struct dsa_private_key *key,
 	    const struct dsa_signature *expected);
 
+#if 0
 void
-test_dsa_key(struct dsa_public_key *pub,
-	     struct dsa_private_key *key,
+test_dsa_sign(const struct dsa_public_key *pub,
+	      const struct dsa_private_key *key,
+	      const struct nettle_hash *hash,
+	      const struct dsa_signature *expected);
+#endif
+
+void
+test_dsa_verify(const struct dsa_params *params,
+		const mpz_t pub,
+		const struct nettle_hash *hash,
+		struct tstring *msg,
+		const struct dsa_signature *ref);
+
+void
+test_dsa_key(const struct dsa_params *params,
+	     const mpz_t pub,
+	     const mpz_t key,
 	     unsigned q_size);
 
 extern const struct ecc_curve * const ecc_curves[];
@@ -229,7 +250,8 @@ test_ecc_mul_j (unsigned curve, unsigned n, const mp_limb_t *p);
 #define ASSERT(x) do {							\
     if (!(x))								\
       {									\
-	fprintf(stderr, "Assert failed %d: %s\n", __LINE__, #x);	\
+	fprintf(stderr, "Assert failed: %s:%d: %s\n", \
+		__FILE__, __LINE__, #x);					\
 	FAIL();								\
       }									\
   } while(0)
